<?php
/**
 * @author	Andrey Chesnakov
 * @link	https://clayrabbit.ru
 */

class ControllerExtensionModuleProductChangeLog extends Controller {

    const FILE = __FILE__;
    const VERSION = '1.2';
    
    private $route;
    private $name;
    private $type;
    private $error = array();
    private $data = array();
    private $token;
    private $settings = array(
        'status'        => 0,
        'language'      => 0,
    );

    public function __construct($registry)
    {
        parent::__construct($registry);
        
        if (!isset($this->data)) {
            $this->data = array();
        }

        $this->type = basename(dirname($this::FILE));
        
        if ((float)VERSION >= 3) {
            $this->route = $this->type . '/' . basename($this::FILE, '.php');
            $this->name = str_replace("/", "_", $this->route);
        } else {
            $this->name = basename($this::FILE, '.php');
            $this->route = $this->type . '/' . $this->name;
        }
        
        if ((float)VERSION >= 2) {
            $this->route = "extension/" . $this->route;
        }

        // Load language and necessary models
        
        if ((float)VERSION >= 3) {
            $this->load->language('common/column_left');
            $this->load->language($this->route);
            $this->token = 'user_token';
        } else {
            if ((float)VERSION >= 2) {
                $this->data += $this->load->language('common/column_left');
            }
            $this->data += $this->load->language($this->route);
            $this->token = 'token';
        }
        
        // Load settings
        foreach ($this->settings as $key => $default) {
            $realkey = $this->name . '_' . $key;
            $this->settings[$realkey] = $this->config->get($realkey);
            if (is_null($this->settings[$realkey])) {
                $this->settings[$realkey] = $default;
            }
            unset($this->settings[$key]);
        }
    }        

    public function install() {
        $this->db->query("CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "product_change_log` (
          `id` INT(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,
          `date_added` datetime NOT NULL,
          `product_id` int NOT NULL,
          `product_name` varchar(255) NOT NULL,
          `user_id` int NOT NULL,
          `username` varchar(128) NOT NULL,
          `route` varchar(128) NOT NULL,
          `ip` varchar(128) NOT NULL,
          `info` text NOT NULL,
          INDEX `product_id` (`product_id`)
        ) DEFAULT CHARSET=utf8;");
    }

    public function uninstall() {
        $this->load->model('setting/event');
        $this->model_setting_event->deleteEventByCode($this->name . '_mp');
        $this->model_setting_event->deleteEventByCode($this->name . '_mpm');
        $this->model_setting_event->deleteEventByCode($this->name . '_mp_after');
        $this->model_setting_event->deleteEventByCode($this->name . '_mpm_after');
    }

    public function index() {
        $this->data['action'] = $this->url->link($this->route, $this->token . '=' . $this->session->data[$this->token], true);
        $this->data['cancel'] = $this->url->link(((float)VERSION >= 3 ? 'marketplace' : 'extension') . '/extension', $this->token . '=' . $this->session->data[$this->token] . '&type=' . $this->type, true);
        
        if ($this->request->server['REQUEST_METHOD'] == 'POST' AND $this->validate()) {
            
            foreach ($this->settings AS $key => $value) {
                if (isset($this->request->post[$key])) {
                    $this->settings[$key] = $this->request->post[$key];
                }
			}
            
            if (!empty($this->request->post[$this->name . '_status'])) {
                $this->uninstall();
                $this->model_setting_event->addEvent($this->name . '_mp', 'admin/controller/extension/module/manager_product/edit/before', $this->route . '/eventBeforeManagerProduct');
                $this->model_setting_event->addEvent($this->name . '_mpm', 'admin/controller/extension/module/manager_product/updateProductMassData/before', $this->route . '/eventBeforeManagerProduct');
                $this->model_setting_event->addEvent($this->name . '_mp_after', 'admin/controller/extension/module/manager_product/edit/after', $this->route . '/eventAfterManagerProduct');
                $this->model_setting_event->addEvent($this->name . '_mpm_after', 'admin/controller/extension/module/manager_product/updateProductMassData/after', $this->route . '/eventAfterManagerProduct');                

            }
            
            $this->load->model('setting/setting');
            $this->model_setting_setting->editSetting($this->name, $this->settings);
            
            $this->session->data['success'] = $this->language->get('text_success');

			$this->response->redirect($this->data['action']);
        }
        
        $this->data['heading_title'] = $this->language->get('heading_title');
        $this->document->setTitle($this->data['heading_title']);


        $this->data['breadcrumbs'] = array();

        $this->data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_home'),
            'href' => $this->url->link('common/dashboard', $this->token . '=' . $this->session->data[$this->token], true)
        );

        $this->data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_extension'),
            'href' => $this->data['cancel']
        );

        $this->data['breadcrumbs'][] = array(
            'text' => $this->data['heading_title'],
            'href' => $this->data['action']
        );

        $this->data['name'] = $this->name;
        $this->data['version'] = $this::VERSION;
        $this->data['settings'] = $this->settings;
        $this->data['route'] = $this->route;
        $this->data['user_token'] = $this->session->data[$this->token];
        
        if (isset($this->request->get['filter_name'])) {
            $filter_name = $this->request->get['filter_name'];
        } else {
            $filter_name = '';
        }
        if (isset($this->request->get['page'])) {
            $page = (int)$this->request->get['page'];
        } else {
            $page = 1;
        }
        
        $this->load->model('localisation/language');
        $this->data['languages'] = $this->model_localisation_language->getLanguages();
        $this->data['filter_name'] = $filter_name;
        $this->data['entries'] = array();

        $sql = " FROM `" . DB_PREFIX . "product_change_log` pcl
            LEFT JOIN `" . DB_PREFIX . "product_description` pd USING (product_id)
            WHERE (pd.language_id IS NULL OR pd.language_id = " . (int)$this->config->get('config_language_id') .")";
        if ($filter_name) {
            $sql .= " AND (pcl.product_name = '" . $this->db->escape($filter_name) . "' OR pd.name = '" . $this->db->escape($filter_name) . "')";
        }
        $query = $this->db->query("SELECT COUNT(*) AS total" . $sql);
        $product_total = $query->row['total'];

        if ($product_total) {
            $this->load->model('user/user');
            $query = $this->db->query("SELECT pcl.*, pd.name" . $sql . "
                ORDER BY date_added DESC
                LIMIT " . ($page - 1) * $this->config->get('config_limit_admin') . "," . $this->config->get('config_limit_admin'));
            foreach ($query->rows as $row) {
                $row['date_added'] = date("d.m.Y H:i:s", strtotime($row['date_added']));
                //$row['user_name'] = $this->model_user_user->getUser($row['user_id'])['username'];
                $row['user_name'] = $row['username'];
                $row['changes'] = '';
                foreach (json_decode($row['info'], 1)['changes'] as $k => $v) {
                    $row['changes'] .= $k .": " . $v[0] . ' => ' . $v[1] . '<br>';
                }
                $row['href'] = $this->url->link('catalog/product/edit', $this->token . '=' . $this->session->data['user_token'] . '&product_id=' . $row['product_id'], true);
                $this->data['entries'][] = $row;
            }
        }
        
        $pagination = new Pagination();
        $pagination->total = $product_total;
        $pagination->page = $page;
        $pagination->limit = $this->config->get('config_limit_admin');
        $pagination->url = $this->data['action'] . '&page={page}';
        $this->data['pagination'] = $pagination->render();

        $this->data['results'] = sprintf($this->language->get('text_pagination'), ($product_total) ? (($page - 1) * $this->config->get('config_limit_admin')) + 1 : 0, ((($page - 1) * $this->config->get('config_limit_admin')) > ($product_total - $this->config->get('config_limit_admin'))) ? $product_total : ((($page - 1) * $this->config->get('config_limit_admin')) + $this->config->get('config_limit_admin')), $product_total, ceil($product_total / $this->config->get('config_limit_admin')));

        $this->data['error_warning'] = isset($this->error['warning']) ? $this->error['warning'] : '';

		$this->data['text_enabled'] = $this->language->get('text_enabled');
		$this->data['text_disabled'] = $this->language->get('text_disabled');
        
        $this->data['header'] = $this->load->controller('common/header');
        $this->data['column_left'] = $this->load->controller('common/column_left');
        $this->data['footer'] = $this->load->controller('common/footer');

        $this->response->setOutput($this->load->view($this->route . ((float)VERSION < 2.2 ? '.tpl' : ''), $this->data));
    }

    public function eventBeforeManagerProduct(&$route, &$data) {
        if (!$this->config->get($this->name . '_status')) return;
        
        if ($route == 'extension/module/manager_product/updateProductMassData') {
            $selected = $this->request->post['selected'];
        } else {
            $selected = array($this->request->get['product_id']);
        }
        $query = $this->db->query("SELECT product_id, price, quantity FROM  `" . DB_PREFIX . "product` 
            WHERE product_id IN (" . implode(',', array_map('intval', $selected)) . ")");
        
        $GLOBALS['_products_before'] = array_column($query->rows, null, 'product_id');
    }
    
    public function eventAfterManagerProduct(&$route, &$output) {
        if (!$this->config->get($this->name . '_status') OR empty($GLOBALS['_products_before'])) return;
        
        $query = $this->db->query("SELECT product_id, price, quantity FROM  `" . DB_PREFIX . "product` 
            WHERE product_id IN (" . implode(',', array_map('intval', array_keys($GLOBALS['_products_before']))) . ")");
                
        foreach ($query->rows as $row) {
            $_changes = array();
            if ((float)$row['price'] != (float)$GLOBALS['_products_before'][$row['product_id']]['price']) {
                $_changes['price'] = array($GLOBALS['_products_before'][$row['product_id']]['price'], $row['price']);
            }
            if ((int)$row['quantity'] != (int)$GLOBALS['_products_before'][$row['product_id']]['quantity']) {
                $_changes['quantity'] = array($GLOBALS['_products_before'][$row['product_id']]['quantity'], $row['quantity']);
            }
            if ($_changes) {
                $this->load->model('extension/module/product_change_log');
                $this->model_extension_module_product_change_log->addEntry(array(
                    'product_id' => $row['product_id'],
                    'info' => array(
                        'caller' => $route,
                        'changes' => $_changes,
                    ),
                ));
            }
        }
/*        
        if ($this->config->get('module_product_change_log_status') AND ($this->request->get['type_data'] == 'quantity' OR $this->request->get['type_data'] == 'price')) {
            $_query = $this->db->query("SELECT price, quantity FROM  `" . DB_PREFIX . "product` WHERE product_id = " . (int)$this->request->get['product_id']);
            
            $_changes = array();
            if ($this->request->get['type_data'] == 'price' AND (float)$_query->row['price'] != (float)$this->request->get['value_data']) {
                $_changes['price'] = array($_query->row['price'], (float)$this->request->get['value_data']);
            }
            if ($this->request->get['type_data'] == 'quantity' AND (int)$_query->row['quantity'] != (int)$this->request->get['value_data']) {
                $_changes['quantity'] = array($_query->row['quantity'], (int)$this->request->get['value_data']);
            }
            if ($_changes) {
                $this->load->model('extension/module/product_change_log');
                $this->model_extension_module_product_change_log->addEntry(array(
                    'product_id' => (int)$this->request->get['product_id'],
                    'info' => array(
                        'caller' => basename(__FILE__) . ':' . __FUNCTION__,
                        'changes' => $_changes,
                    ),
                ));
            }
        }
 */
    }
    
    protected function validate() {
        if (!$this->user->hasPermission('modify', $this->route)) {
            $this->error['warning'] = $this->language->get('error_permission');
        }

        return !$this->error;
    }
}
